@@ -1,4 +1,4 @@
-// $Id: tag_file.cpp,v 1.40 2002/07/31 13:45:18 t1mpy Exp $
+// $Id: tag_file.cpp,v 1.42 2002/11/02 21:02:09 t1mpy Exp $
 
 // id3lib: a C++ library for creating and manipulating id3v1/v2 tags
 // Copyright 1999, 2000  Scott Thomas Haug
@@ -50,7 +50,7 @@ static int truncate(const char *path, size_t length)
 {
   int result = -1;
   HANDLE fh;
-  
+
   fh = ::CreateFile(path,
                     GENERIC_WRITE | GENERIC_READ,
                     0,
@@ -58,7 +58,7 @@ static int truncate(const char *path, size_t length)
                     OPEN_EXISTING,
                     FILE_ATTRIBUTE_NORMAL,
                     NULL);
-  
+
   if(INVALID_HANDLE_VALUE != fh)
   {
     SetFilePointer(fh, length, NULL, FILE_BEGIN);
@@ -66,7 +66,7 @@ static int truncate(const char *path, size_t length)
     CloseHandle(fh);
     result = 0;
   }
-  
+
   return result;
 }
 
@@ -76,7 +76,7 @@ static int truncate(const char *path, size_t length)
 #  endif
 
 #elif defined(WINCE)
-// Createfile is apparently to defined to CreateFileW. (Bad Bad Bad), so we 
+// Createfile is apparently to defined to CreateFileW. (Bad Bad Bad), so we
 // work around it by converting path to Unicode
 #  include <windows.h>
 static int truncate(const char *path, size_t length)
@@ -100,7 +100,7 @@ static int truncate(const char *path, size_t length)
     CloseHandle(fh);
     result = 0;
   }
-  
+
   return result;
 }
 
@@ -131,7 +131,7 @@ size_t ID3_TagImpl::Link(const char *fileInfo, bool parseID3v1, bool parseLyrics
 size_t ID3_TagImpl::Link(const char *fileInfo, flags_t tag_types)
 {
   _tags_to_parse.set(tag_types);
-  
+
   if (NULL == fileInfo)
   {
     return 0;
@@ -141,7 +141,20 @@ size_t ID3_TagImpl::Link(const char *fileInfo, flags_t tag_types)
   _changed = true;
 
   this->ParseFile();
-  
+
+  return this->GetPrependedBytes();
+}
+
+// used for streaming:
+size_t ID3_TagImpl::Link(ID3_Reader &reader, flags_t tag_types)
+{
+  _tags_to_parse.set(tag_types);
+
+  _file_name = "";
+  _changed = true;
+
+  this->ParseReader(reader);
+
   return this->GetPrependedBytes();
 }
 
@@ -176,16 +189,16 @@ size_t RenderV1ToFile(ID3_TagImpl& tag, fstream& file)
     {
       file.seekp(0-ID3_V1_LEN, ios::end);
     }
-    // Otherwise, set the cursor to the end of the file so we can append on 
+    // Otherwise, set the cursor to the end of the file so we can append on
     // the new tag.
     else
     {
       file.seekp(0, ios::end);
     }
   }
-  
+
   ID3_IOStreamWriter out(file);
-  
+
   id3::v1::render(out, tag);
 
   return ID3_V1_LEN;
@@ -200,7 +213,7 @@ size_t RenderV2ToFile(const ID3_TagImpl& tag, fstream& file)
     return 0;
   }
 
-  String tagString; 
+  String tagString;
   io::StringWriter writer(tagString);
   id3::v2::render(writer, tag);
   ID3D_NOTICE( "RenderV2ToFile: rendered v2" );
@@ -243,13 +256,13 @@ size_t RenderV2ToFile(const ID3_TagImpl& tag, fstream& file)
       size_t nBytes = file.gcount();
       tmpOut.write((char *)tmpBuffer, nBytes);
     }
-      
+
 #else //((defined(__GNUC__) && __GNUC__ >= 3  ) || !defined(HAVE_MKSTEMP))
 
     // else we gotta make a temp file, copy the tag into it, copy the
     // rest of the old file after the tag, delete the old file, rename
     // this new file to the old file's name and update the handle
-    
+
     int fd = mkstemp(sTempFile);
     if (fd < 0)
     {
@@ -276,7 +289,7 @@ size_t RenderV2ToFile(const ID3_TagImpl& tag, fstream& file)
       size_t nBytes = file.gcount();
       tmpOut.write(tmpBuffer, nBytes);
     }
-      
+
 #endif ////((defined(__GNUC__) && __GNUC__ >= 3  ) || !defined(HAVE_MKSTEMP))
 
     tmpOut.close();
@@ -297,6 +310,7 @@ size_t RenderV2ToFile(const ID3_TagImpl& tag, fstream& file)
 #endif //defined(HAVE_SYS_STAT_H)
 
 //    file = tmpOut;
+    file.clear();//to clear the eof mark
     openWritableFile(filename, file);
   }
 
@@ -307,12 +321,12 @@ size_t RenderV2ToFile(const ID3_TagImpl& tag, fstream& file)
 flags_t ID3_TagImpl::Update(flags_t ulTagFlag)
 {
   flags_t tags = ID3TT_NONE;
-  
+
   fstream file;
   String filename = this->GetFileName();
   ID3_Err err = openWritableFile(filename, file);
   _file_size = getFileSize(file);
-  
+
   if (err == ID3E_NoFile)
   {
     err = createFile(filename, file);
@@ -330,8 +344,8 @@ flags_t ID3_TagImpl::Update(flags_t ulTagFlag)
       tags |= ID3TT_ID3V2;
     }
   }
-  
-  if ((ulTagFlag & ID3TT_ID3V1) && 
+
+  if ((ulTagFlag & ID3TT_ID3V1) &&
       (!this->HasTagType(ID3TT_ID3V1) || this->HasChanged()))
   {
     size_t tag_bytes = RenderV1ToFile(*this, file);
@@ -356,7 +370,7 @@ flags_t ID3_TagImpl::Strip(flags_t ulTagFlag)
 {
   flags_t ulTags = ID3TT_NONE;
   const size_t data_size = ID3_GetDataSize(*this);
-  
+
   // First remove the v2 tag, if requested
   if (ulTagFlag & ID3TT_PREPENDED & _file_tags.get())
   {
@@ -372,9 +386,9 @@ flags_t ID3_TagImpl::Strip(flags_t ulTagFlag)
     // tag back n bytes, where n is the size of the id3v2 tag.  Once we've
     // copied the data, we'll truncate the file.
     file.seekg(this->GetPrependedBytes(), ios::beg);
-    
+
     uchar aucBuffer[BUFSIZ];
-    
+
     // The nBytesRemaining variable indicates how many bytes are to be copied
     size_t nBytesToCopy = data_size;
 
@@ -384,12 +398,12 @@ flags_t ID3_TagImpl::Strip(flags_t ulTagFlag)
     {
       nBytesToCopy += this->GetAppendedBytes();
     }
-    
-    // The nBytesRemaining variable indicates how many bytes are left to be 
+
+    // The nBytesRemaining variable indicates how many bytes are left to be
     // moved in the actual file.
     // The nBytesCopied variable keeps track of how many actual bytes were
     // copied (or moved) so far.
-    size_t 
+    size_t
       nBytesRemaining = nBytesToCopy,
       nBytesCopied = 0;
     while (!file.eof())
@@ -416,7 +430,7 @@ flags_t ID3_TagImpl::Strip(flags_t ulTagFlag)
         file.seekg(this->GetPrependedBytes(), ios::cur);
         nBytesCopied += nBytesRead;
       }
-      
+
       if (nBytesCopied == nBytesToCopy || nBytesToRead < BUFSIZ)
       {
         break;
@@ -424,7 +438,7 @@ flags_t ID3_TagImpl::Strip(flags_t ulTagFlag)
     }
     file.close();
   }
-  
+
   size_t nNewFileSize = data_size;
 
   if ((_file_tags.get() & ID3TT_APPENDED) && (ulTagFlag & ID3TT_APPENDED))
@@ -437,7 +451,7 @@ flags_t ID3_TagImpl::Strip(flags_t ulTagFlag)
     // size by those bytes
     nNewFileSize += this->GetAppendedBytes();
   }
-  
+
   if ((ulTagFlag & ID3TT_PREPENDED) && (_file_tags.get() & ID3TT_PREPENDED))
   {
     // If we're stripping the ID3v2 tag, there's no need to adjust the new
@@ -462,9 +476,9 @@ flags_t ID3_TagImpl::Strip(flags_t ulTagFlag)
   _prepended_bytes = (ulTags & ID3TT_PREPENDED) ? 0 : _prepended_bytes;
   _appended_bytes  = (ulTags & ID3TT_APPENDED)  ? 0 : _appended_bytes;
   _file_size = data_size + _prepended_bytes + _appended_bytes;
-  
+
   _changed = _file_tags.remove(ulTags) || _changed;
-  
+
   return ulTags;
 }
 
